// Copyright (c) 2000-2001 Brad Hughes <bhughes@trolltech.com>
//
// Use, modification and distribution is allowed without limitation,
// warranty, or liability of any kind.
//

#include <QtGui>
#include <QObject>
#include <QStringList>
#include <QApplication>
#include <QApplication>

#include "output.h"
#include "visual.h"

#include <stdio.h>

// static methods

static QList<OutputFactory*> *factories = 0;
static QStringList files;

static void checkFactories()
{
   if ( ! factories )
   {
      files.clear();
      factories = new QList<OutputFactory *>;

      QDir pluginsDir ( qApp->applicationDirPath() );
      pluginsDir.cdUp();
      pluginsDir.cd ( "Plugins/Output" );
      foreach ( QString fileName, pluginsDir.entryList ( QDir::Files ) )
      {
         QPluginLoader loader ( pluginsDir.absoluteFilePath ( fileName ) );
         QObject *plugin = loader.instance();
         if ( loader.isLoaded() )
         {
            qDebug ( "Output: plugin loaded - %s", qPrintable ( fileName ) );
         }
         OutputFactory *factory = 0;
         if ( plugin )
            factory = qobject_cast<OutputFactory *> ( plugin );

         if ( factory )
         {
            Output::registerFactory ( factory );
            files << pluginsDir.absoluteFilePath(fileName);
         }
      }
   }
}

void Output::registerFactory ( OutputFactory *fact )
{
   factories->append ( fact );
}

Output *Output::create ( QObject *parent )
{
   Output *output = 0;

   checkFactories();
   OutputFactory *fact = factories->at ( 0 ); //TODO add factories support
   if ( fact )
   {
      output = fact->create ( parent );
   }
   return output;
}

QList<OutputFactory*> *Output::outputFactories()
{
   checkFactories();
   return factories;
}

QStringList Output::outputFiles()
{
    checkFactories();
    return files;
}

Output::Output ( QObject* parent ) :  QThread ( parent ), r ( stackSize() ), bufsize ( stackSize () )
{}


Output::~Output()
{}


void Output::addListener ( QObject *o )
{
   if ( listeners.indexOf ( o ) == -1 )
      listeners.append ( o );
}


void Output::removeListener ( QObject *o )
{
   listeners.removeAll ( o );
}


/*void Output::dispatch(OutputEvent &e)
{
    QObject *object = listeners.first();
    while (object) {
   QApplication::postEvent(object, new OutputEvent(e));
   object = listeners.next();
    }
}*/


void Output::error ( const QString &e )
{
   /*QObject *object = listeners.first();
   while (object) {
   QApplication::postEvent(object, new OutputEvent(e));
   object = listeners.next();
   }*/
   //     m_stat = OutputStatus(e);
   //     emit statusChanged();
   dispatch ( OutputState ( e ) );
}


void Output::addVisual ( Visual *v )
{
   if ( visuals.indexOf ( v ) == -1 )
   {
      visuals.append ( v );


   }
}


void Output::removeVisual ( Visual *v )
{
   visuals.removeAll ( v );
}

void Output::dispatchVisual ( Buffer *buffer, unsigned long written,
                              int chan, int prec )
{
   if ( ! buffer || !visuals.size())
      return;
   Visual *visual = 0;
   foreach (visual , visuals );
   {
      visual->mutex()->lock ();
      visual->add ( buffer, written, chan, prec );
      visual->mutex()->unlock();
   }
}


void Output::prepareVisuals()
{
   Visual *visual = visuals.first();
   /*while (visual) {
   visual->mutex()->lock();
   visual->prepare();
   visual->mutex()->unlock();

   visual = visuals.next();
   }*/
}

void Output::dispatch ( const OutputState &st )
{
   m_state = st;
   emit stateChanged ( &m_state );
}

