/***************************************************************************
 *   Copyright (C) 2006 by Ilya Kotov                                      *
 *   forkotov02@hotmail.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QPixmap>
#include <QResizeEvent>
#include <QPainter>
#include <QFont>
#include <QFontMetrics>
#include <QSettings>
#include <QMenu>

#include "mediafile.h"
#include "textscroller.h"
#include "listwidget.h"
#include "skin.h"
#include "playlistmodel.h"
#include "playlist.h"

ListWidget::ListWidget(QWidget *parent)
        : QWidget(parent)
{
    m_update = FALSE;
    m_skin = Skin::getPointer();
    loadColors();
    setWindowFlags(Qt::FramelessWindowHint);
    m_menu = new QMenu(this);

    m_first = 0;
    m_rows = 0;
    m_scroll = FALSE;
    readSettings();
    connect(m_skin, SIGNAL(skinChanged()), this, SLOT(updateSkin()));
    setAutoFillBackground(TRUE);
}


ListWidget::~ListWidget()
{}

void ListWidget::readSettings()
{
    QSettings settings(QDir::homePath()+"/.qmmp/qmmprc", QSettings::IniFormat);
    QString fontname = settings.value("PlayList/Font","").toString();
    if (fontname.isEmpty ())
        fontname = QFont("Helvetica [Cronyx]", 10).toString();
    m_font.fromString(fontname);

    if(m_update)
    {
        delete m_metrics;
        m_metrics = new QFontMetrics(m_font);
        m_rows = (height() - 10) / m_metrics->ascent ();
        updateList();
    }
    else
    {
        m_update = TRUE;
        m_metrics = new QFontMetrics(m_font);
    }
}

void ListWidget::loadColors()
{
    m_normal.setNamedColor(m_skin->getPLValue("normal"));
    m_current.setNamedColor(m_skin->getPLValue("current"));
    m_normal_bg.setNamedColor(m_skin->getPLValue("normalbg"));
    m_selected_bg.setNamedColor(m_skin->getPLValue("selectedbg"));
}

void ListWidget::paintEvent(QPaintEvent *)
{

    QPainter m_painter(this);
    //m_painter.setPen(Qt::white);
    m_painter.setFont(m_font);
    m_painter.setBrush(QBrush(m_normal_bg));
    m_painter.drawRect(-1,-1,width()+1,height()+1);



    for (int i=0; i<m_titles.size(); ++i )
    {
        if (m_model->isSelected(i + m_first))
        {
            m_painter.setBrush(QBrush(m_selected_bg));
            m_painter.setPen(m_selected_bg);
            m_painter.drawRect ( 6, 15+(i-1)*m_metrics->ascent(),
                                 width() - 10, m_metrics->ascent());
        }

        if (m_model->currentRow() == i + m_first)
            m_painter.setPen(m_current);
        else
            m_painter.setPen(m_normal);  //243,58

        m_painter.drawText(10,14+i*m_metrics->ascent(),m_titles.at(i));
        m_painter.drawText(width() - 7 - m_metrics->width(m_times.at(i)),
                           14+i*m_metrics->ascent (), m_times.at(i));

    }

}

void ListWidget::mouseDoubleClickEvent (QMouseEvent *e)
{
    int y = e->y();
    for (int i = 0; i<m_rows; ++i )
    {
        if ((y > 2+i*m_metrics->ascent())&&(y < 14+i*m_metrics->ascent()))
        {
            m_model->setCurrent(m_first+i);
            emit selectionChanged();
            update();
            return;
        }
    }
}

void ListWidget::mousePressEvent(QMouseEvent *e)
{
    m_scroll = TRUE;
    if (e->button () == Qt::RightButton)
    {
        m_menu->exec(e->globalPos());
        return;
    }
    int y = e->y();   
    if(!Qt::ControlModifier == e->modifiers ())
        m_model->clearSelection();
    for (int i = 0; i < qMin(m_rows, m_model->count() - m_first); ++i )
    {
        if ((y > 2+i*m_metrics->ascent())&&(y < 14+i*m_metrics->ascent()))
        {
            m_model->setSelected(m_first+i, !m_model->isSelected(m_first+i));
            update();
            return;
        }
    }
}

void ListWidget::resizeEvent(QResizeEvent *e)
{

    m_rows = (e->size().height() - 10) / m_metrics->ascent ();
    if (m_first > 0)
        m_first = qMax ( m_model->currentRow() - m_rows/2 + 1, 0);

    if (m_model->count() < (m_rows+m_first+1) && m_rows<m_model->count())
    {
        m_first = m_model->count() - m_rows;
    }
    else
        m_first = 0;
    updateList();
}

void ListWidget::wheelEvent (QWheelEvent *e)
{
    if (m_model->count() <= m_rows)
        return;
    if((m_first == 0 && e->delta() > 0) ||
            ((m_first == m_model->count() - m_rows) && e->delta() < 0))
        return;
    m_first -= e->delta()/40;  //40*3 TODO: add step to config
    if(m_first < 0)  
       m_first = 0;
    
    if(m_first > m_model->count() - m_rows)
       m_first = m_model->count() - m_rows;

    m_scroll = TRUE;
    updateList();
}

void ListWidget::updateList()
{
    /*if(m_model->count() == 0)
        return;*/
    if(!m_scroll)
    {
        if(m_first + m_rows < m_model->currentRow() + 1)
            m_first = qMin(m_model->count() - m_rows,
                           m_model->currentRow() - m_rows/2 + 1);
        else if(m_first > m_model->currentRow())
            m_first = qMax (m_model->currentRow() - m_rows/2 + 1, 0);
    }

    if (m_model->count() <= m_rows)
        emit positionChanged(0);
    else
    {
        int pos = m_first*99/(m_model->count() - m_rows);
        emit positionChanged(pos);
    }

    m_titles = m_model->getTitles(m_first, m_rows );
    m_times  = m_model->getTimes(m_first, m_rows );
    m_scroll = FALSE;
    //add numbers
    for(int i = 0; i < m_titles.size(); ++i)
    {
        QString title = m_titles.at(i);
        m_titles.replace(i, title.prepend(QString("%1").arg(m_first+i+1)+". "));

    }
    if(m_model->currentItem())
    {
       TextScroller::getPointer()->setText("***  "+m_model->currentItem()->title());
       parentWidget()->parentWidget()->setWindowTitle(m_model->currentItem()->title());
    }
    cut();
    update();
}

void ListWidget::setModel(PlayListModel *model)
{
    m_model = model;
    connect (m_model, SIGNAL(listChanged()), SLOT(updateList()));
    //connect (m_detailsAct, SIGNAL(triggered()), model, SLOT(showDetails ()));
    updateList();
}

void ListWidget::scroll(int sc)
{

    if (m_model->count() <= m_rows)
        return;
    m_first = sc*(m_model->count() - m_rows)/99;
    m_scroll = TRUE;
    updateList();

}

void ListWidget::cut()
{
    bool cut;
    for (int i=0; i<m_titles.size(); ++i )
    {
        QString name;
        cut = FALSE;
        while( m_metrics->width(m_titles.at(i)) > (this->width() - 54))
        {
            cut = TRUE;
            name = m_titles.at(i);
            m_titles.replace(i, name.left(name.length()-1) );
        }
        if (cut)
        {
            m_titles.replace(i, name.left(name.length()-3).trimmed()+"...");

        }
    }

}

void ListWidget::updateSkin()
{
    loadColors();
    update();
}

