/***************************************************************************
 *   Copyright (C) 2006 by Ilya Kotov                                      *
 *   forkotov02@hotmail.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QWidget>
#include <QFile>
#include <QDir>

#include <decoder.h>
#include <factory.h>

#include "playlistmodel.h"
#include "mediafile.h"

PlayListModel::PlayListModel ( QObject *parent )
      : QObject ( parent )
{
   readSettings();
   m_current = 0;
}

PlayListModel::~PlayListModel()
{
   writeSettings();
   clear();
}

void PlayListModel::load ( MediaFile *file )
{
   m_files << file;
   emit listChanged();
}

int PlayListModel::count()
{
   return m_files.size();
}

MediaFile* PlayListModel::currentItem()
{
   if ( m_files.isEmpty() )
      return 0;
   else
      return m_files.at ( m_current );
}

int PlayListModel::currentRow()
{
   return m_current;
}

bool PlayListModel::setCurrent ( int c )
{
   if ( c > count()-1 )
      return FALSE;
   m_current = c;
   emit listChanged();
   return TRUE;
}

bool PlayListModel::next()
{
   if ( m_current == count()-1 )
      return FALSE;
   m_current++;
   emit listChanged();
   return TRUE;

}

bool PlayListModel::previous()
{
   if ( m_current < 1 )
      return FALSE;
   m_current--;
   emit listChanged();
   return TRUE;
}

void PlayListModel::clear()
{
   m_current = 0;
   while ( !m_files.isEmpty() )
      delete m_files.takeFirst();
   emit listChanged();
}

void PlayListModel::clearSelection()
{
   for ( int i = 0; i<m_files.size(); ++i )
      m_files.at ( i )->setSelected ( FALSE );
   emit listChanged();
}

QList <QString> PlayListModel::getTitles ( int b,int l )
{
   QList <QString> m_titles;
   for ( int i = b; ( i < b + l ) && ( i < m_files.size() ); ++i )
      m_titles << m_files.at ( i )->title();
   return m_titles;
}

QList <QString> PlayListModel::getTimes ( int b,int l )
{
   QList <QString> m_times;
   for ( int i = b; ( i < b + l ) && ( i < m_files.size() ); ++i )
      m_times << QString ( "%1" ).arg ( m_files.at ( i )->length() /60 ) +":"
      +QString ( "%1" ).arg ( m_files.at ( i )->length() %60/10 ) +
      QString ( "%1" ).arg ( m_files.at ( i )->length() %60%10 );
   return m_times;
}

bool PlayListModel::isSelected ( int row )
{
   return m_files.at ( row )->isSelected();
}

void PlayListModel::setSelected ( int row, bool yes )
{
   m_files.at ( row )->setSelected ( yes );
}

void PlayListModel::removeSelected()
{
   int i = 0;
   while ( !m_files.isEmpty() &&i<m_files.size() )
   {
      if ( m_files.at ( i )->isSelected() )
      {
         delete m_files.takeAt ( i );
         if ( m_current >= i && m_current!=0 )
            m_current--;
      }
      else
         i++;
   }
   emit listChanged();
}

void PlayListModel::removeUnselected()
{
   int i = 0;
   while ( !m_files.isEmpty() &&i<m_files.size() )
   {
      if ( !m_files.at ( i )->isSelected() )
      {
         delete m_files.takeAt ( i );
         if ( m_current >= i && m_current!=0 )
            m_current--;
      }
      else
         i++;
   }
   emit listChanged();
}

void PlayListModel::invertSelection()
{
   for ( int i = 0; i<m_files.size(); ++i )
      m_files.at ( i )->setSelected ( !m_files.at ( i )->isSelected() );
   emit listChanged();
}

void PlayListModel::selectAll()
{
   for ( int i = 0; i<m_files.size(); ++i )
      m_files.at ( i )->setSelected ( TRUE );
   emit listChanged();
}

void PlayListModel::showDetails()
{
   for ( int i = 0; i<m_files.size(); ++i )
   {
      if ( m_files.at ( i )->isSelected() )
      {
         DecoderFactory *fact = Decoder::findFactory ( m_files.at ( i )->path() );
         if ( fact )
            fact->showDetails ( 0, m_files.at ( i )->path() );

         return;
      }
   }

}

void PlayListModel::readSettings()
{
   QFile file ( QDir::homePath() +"/.qmmp/playlist.txt" );
   file.open ( QIODevice::ReadOnly );
   file.setTextModeEnabled ( TRUE );
   QByteArray line;
   while ( !file.atEnd () )
   {
      line = file.readLine();
      m_files << new MediaFile ( QString::fromUtf8 ( line ).trimmed () );
   }
   file.close ();
}

void PlayListModel::writeSettings()
{
   QFile file ( QDir::homePath() +"/.qmmp/playlist.txt" );
   file.open ( QIODevice::WriteOnly );
   foreach ( MediaFile* m, m_files )
   file.write ( m->path().toUtf8 () +"\n" );
   file.close ();
}

